package com.airfrance.squalecommon.datatransfertobject.stats;

import java.util.Date;

/**
 * Permet de rcuprer les statistiques par application
 */
public class ApplicationStatsDTO {
    /** Le nom de l'application concerne */
    private String mApplicationName;
    /** Indique si l'application est valide */
    private boolean mValidatedApplication;
    /** true si le derniers audits excuts est russis */
    private boolean mLastAuditIsTerminated;
    /** false si aucun audit russis prsent dans les n derniers jours (90 par dfaut) */
    private boolean mActivatedApplication;
    /** date du dernier audit russis */
    private Date mLastTerminatedAuditDate;
    /** la dure en hh:mm du dernier audit */
    private String mLastAuditDuration;
    /** le nombre d'audit ( quelque soit leur tat ) raliss dutant les n derniers jours (10 par dfaut) */
    private int mNbAudits;
    /** le nombre audits en russis */
    private int mNbTerminatedAudits;
    /** le nombre audits partiel ou en chec */
    private int mNbPartialOrFaliedAudits;
    /** la date du dernier audit en chec */
    private Date mLastFailedAuditDate;
    /** la date du plus ancien audit russis */
    private Date mFirstTerminatedAuditDate;
    /** le nom du serveur de l'application */
    private String mServerName;
    /** la frquence de purge */
    private String mPurgeFrequency = "-";
    /** Date du dernier accs utilisateur */
    private Date mLastAccess;

    /**
     * @return false si aucun audit russis prsent dans les n derniers jours
     */
    public boolean isActivatedApplication() {
        return mActivatedApplication;
    }

    /**
     * @return le nom de l'application concerne
     */
    public String getApplicationName() {
        return mApplicationName;
    }

    /**
     * @return la date du plus ancien audit russis
     */
    public Date getFirstTerminatedAuditDate() {
        return mFirstTerminatedAuditDate;
    }

    /**
     * @return la date du dernier accs utilisateur
     */
    public Date getLastAccess() {
        return mLastAccess;
    }

    /**
     * @return la dure en hh:mm du dernier audit
     */
    public String getLastAuditDuration() {
        return mLastAuditDuration;
    }

    /**
     * @return la date du dernier audit en chec
     */
    public Date getLastFailedAuditDate() {
        return mLastFailedAuditDate;
    }

    /**
     * @return la date du dernier audit russis
     */
    public Date getLastTerminatedAuditDate() {
        return mLastTerminatedAuditDate;
    }

    /**
     * @return le nombre d'audit ( quelque soit leur tat ) raliss dutant les n derniers jours
     */
    public int getNbAudits() {
        return mNbAudits;
    }

    /**
     * @return le nombre audits partiel ou en chec
     */
    public int getNbPartialOrFaliedAudits() {
        return mNbPartialOrFaliedAudits;
    }

    /**
     * @return le nombre audits en russis
     */
    public int getNbTerminatedAudits() {
        return mNbTerminatedAudits;
    }

    /**
     * @return le nom du serveur de l'application
     */
    public String getServerName() {
        return mServerName;
    }

    /**
     * @return true si le derniers audits excuts est russis
     */
    public boolean isLastAuditIsTerminated() {
        return mLastAuditIsTerminated;
    }

    /**
     * @return true si l'application est valide
     */
    public boolean isValidatedApplication() {
        return mValidatedApplication;
    }

    /**
     * @param pActivated false si aucun audit russis prsent dans les n derniers jours
     */
    public void setActivatedApplication(boolean pActivated) {
        mActivatedApplication = pActivated;
    }

    /**
     * @param pName le nom de l'application
     */
    public void setApplicationName(String pName) {
        mApplicationName = pName;
    }

    /**
     * @param pDate la date du plus ancien audit russis
     */
    public void setFirstTerminatedAuditDate(Date pDate) {
        mFirstTerminatedAuditDate = pDate;
    }

    /**
     * @param pDate la date du dernier accs utilisateur
     */
    public void setLastAccess(Date pDate) {
        mLastAccess = pDate;
    }

    /**
     * @param pDuration la dure en hh:mm du dernier audit
     */
    public void setLastAuditDuration(String pDuration) {
        mLastAuditDuration = pDuration;
    }

    /**
     * @param pDate la date du dernier audit en chec
     */
    public void setLastFailedAuditDate(Date pDate) {
        mLastFailedAuditDate = pDate;
    }

    /**
     * @param pDate la date du dernier audit russis
     */
    public void setLastTerminatedAuditDate(Date pDate) {
        mLastTerminatedAuditDate = pDate;
    }

    /**
     * @param pNbAudits le nombre d'audit ( quelque soit leur tat ) raliss dutant les n derniers jours
     */
    public void setNbAudits(int pNbAudits) {
        mNbAudits = pNbAudits;
    }

    /**
     * @param pNbAudits le nombre audits partiel ou en chec
     */
    public void setNbPartialOrFaliedAudits(int pNbAudits) {
        mNbPartialOrFaliedAudits = pNbAudits;
    }

    /**
     * @param pNbAudits le nombre audits en russis
     */
    public void setNbTerminatedAudits(int pNbAudits) {
        mNbTerminatedAudits = pNbAudits;
    }

    /**
     * @param pServerName le nom du serveur de l'application
     */
    public void setServerName(String pServerName) {
        mServerName = pServerName;
    }

    /**
     * @param pTerminated true si le derniers audit excut est russi
     */
    public void setLastAuditIsTerminated(boolean pTerminated) {
        mLastAuditIsTerminated = pTerminated;
    }

    /**
     * @param pValidated true si l'application est valide
     */
    public void setValidatedApplication(boolean pValidated) {
        mValidatedApplication = pValidated;
    }
    /**
     * @return la frquence de purge
     */
    public String getPurgeFrequency() {
        return mPurgeFrequency;
    }

    /**
     * @param pFreq la frquence de purge
     */
    public void setPurgeFrequency(int pFreq) {
        if(pFreq >= 0) {
            mPurgeFrequency = "" + pFreq;
        }
    }

}
