package com.airfrance.squalecommon.daolayer.message;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafPersistenceException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.SqualeTestCase;
import com.airfrance.squalecommon.enterpriselayer.businessobject.message.MessageBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.message.MessageId;
import com.airfrance.squalecommon.enterpriselayer.businessobject.message.NewsBO;

/**
 * 
 */
public class NewsDAOImplTest extends SqualeTestCase {

    /** le bo */
    private NewsBO bo;

    /** un message BO pour tester */
    private MessageBO messBo;

    /** provider de persistence */
    private static IPersistenceProvider PERSISTENT_PROVIDER;

    /**
     * setUp le test
     * @throws Exception en cas d'chec
     */
    public void setUp() throws Exception {
        super.setUp();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH, Calendar.AUGUST);
        calendar.set(Calendar.YEAR, 2005);
        Date begin = calendar.getTime();
        calendar.set(Calendar.YEAR, 2007);
        Date end = calendar.getTime();
        bo = new NewsBO();
        bo.setId(1);
        bo.setBeginningDate(begin);
        bo.setEndDate(end);
        bo.setKey("test");
        messBo = new MessageBO();
        messBo.setKey("rule.test");
        messBo.setText("Text");
        messBo.setTitle("title");
        messBo.setLang("en");
        MessageId id = new MessageId();
        id.setKey(messBo.getKey());
        id.setLang(messBo.getLang());
        messBo.setId(id);

    }

    /**
     * Test la cration et l'enregistrement en base d'un newsBO et de son messageBO
     *
     */
    public void testAddNews() {
        PERSISTENT_PROVIDER = PersistenceHelper.getPersistenceProvider();
        ISession session;
        NewsDAOImpl dao = NewsDAOImpl.getInstance();
        try {
            session = PERSISTENT_PROVIDER.getSession();
            session.beginTransaction();
            Collection coll = dao.findAll(session);
            // on vrifie qu'on en rcupre bien 0 car pour l'instant il n'y en a pas en session
            assertEquals(coll.size(), 0);
            // on en sauvegarde 1
            dao.addNews(session, bo, messBo);
            session.commitTransactionWithoutClose();
            // aprs avoir commit, on vrifie qu'il a bien t
            // enregistr et qu'on peut le rcuprer
            session.beginTransaction();
            coll = dao.findAll(session);
            session.commitTransaction();
            // vrification
            assertEquals(coll.size(), 1);
        } catch (JrafPersistenceException e) {
            fail("unexpectedException");
            e.printStackTrace();
        } catch (JrafDaoException e) {
            fail("unexpectedException");
            e.printStackTrace();
        }
    }

    /**
     * Test la cration et l'enregistrement en base d'un newsBO et de son messageBO
     *
     */
    public void testModifyNews() {
        PERSISTENT_PROVIDER = PersistenceHelper.getPersistenceProvider();
        ISession session;
        NewsDAOImpl dao = NewsDAOImpl.getInstance();
        try {
            session = PERSISTENT_PROVIDER.getSession();
            session.beginTransaction();
            Collection coll = dao.findAll(session);
            // on vrifie qu'on en rcupre bien 0 car pour l'instant il n'y en a pas en session
            assertEquals(coll.size(), 0);
            // on en sauvegarde 1
            dao.addNews(session, bo, messBo);
            session.commitTransactionWithoutClose();
            // aprs avoir commit, on vrifie qu'il a bien t
            // enregistr et qu'on peut le rcuprer
            session.beginTransaction();
            coll = dao.findAll(session);
            session.commitTransactionWithoutClose();
            // vrification
            assertEquals(coll.size(), 1);
            //Modification
            session.beginTransaction();
            bo.setKey("test.modification");
            messBo.setKey("test.modification.messageKey");
            dao.modifyNews(session, bo, messBo);
            session.commitTransactionWithoutClose();
            session.beginTransaction();
            coll = dao.findAll(session);
            session.commitTransaction();
            // On ne doit toujours qu'en avoir qu'un
            // le meme mais avec la cl modifie
            assertEquals(coll.size(), 1);
            Iterator it = coll.iterator();
            NewsBO newBo = (NewsBO) it.next();
            // on vrifie que la cl a bien t modifie
            assertEquals(newBo.getKey(), "test.modification");
        } catch (JrafPersistenceException e) {
            fail("unexpectedException");
            e.printStackTrace();
        } catch (JrafDaoException e) {
            fail("unexpectedException");
            e.printStackTrace();
        }
    }

    /**
     * Test la cration et l'enregistrement en base d'un newsBO et de son messageBO
     *
     */
    public void testPurgeNews() {
        PERSISTENT_PROVIDER = PersistenceHelper.getPersistenceProvider();
        ISession session;
        NewsDAOImpl dao = NewsDAOImpl.getInstance();
        try {
            session = PERSISTENT_PROVIDER.getSession();
            session.beginTransaction();
            Collection coll = dao.findAll(session);
            // on vrifie qu'on en rcupre bien 0 car pour l'instant il n'y en a pas en session
            assertEquals(coll.size(), 0);
            // on en sauvegarde 1
            dao.addNews(session, bo, messBo);
            session.commitTransactionWithoutClose();
            // aprs avoir commit, on vrifie qu'il a bien t
            // enregistr et qu'on peut le rcuprer
            session.beginTransaction();
            coll = dao.findAll(session);
            session.commitTransactionWithoutClose();
            // vrification
            assertEquals(coll.size(), 1);
            Iterator it = coll.iterator();
            NewsBO newBo = (NewsBO) it.next();
            session.beginTransaction();
            dao.removeNews(session, newBo,messBo);
            session.commitTransactionWithoutClose();
            // on vrifie que la suppression a bien t faite
            session.beginTransaction();
            coll = dao.findAll(session);
            session.commitTransaction();
            // vrification
            assertEquals(coll.size(), 0);

        } catch (JrafPersistenceException e) {
            fail("unexpectedException");
            e.printStackTrace();
        } catch (JrafDaoException e) {
            fail("unexpectedException");
            e.printStackTrace();
        }
    }

    /**
     * Test la rcupration en base de news en fonction de leurs catgories
     * (toutes, les courantes, les anciennes)
     *
     */
    public void testFindWhereKind() {
        PERSISTENT_PROVIDER = PersistenceHelper.getPersistenceProvider();
        ISession session;
        NewsDAOImpl dao = NewsDAOImpl.getInstance();
        MessageDAOImpl messDao = MessageDAOImpl.getInstance();
        Collection coll;
        try {
            session = PERSISTENT_PROVIDER.getSession();
            session.beginTransaction();
            // on en sauvegarde 1
            dao.addNews(session, bo, messBo);
            session.commitTransactionWithoutClose();
            session.beginTransaction();
            coll = dao.findWhereKind(session, "all");
            session.commitTransactionWithoutClose();
            assertTrue(coll.size() > 0);
            /* le todate ne marche pas en test unitaire 
            on ne peut pas effectuer les tests suivants 
            session.beginTransaction();
            coll = dao.findWhereKind(session, "old");
            session.commitTransactionWithoutClose();
            assertTrue(coll.size() == 0);
            session.beginTransaction();
            coll = dao.findWhereKind(session, "current");
            session.commitTransaction();
            assertTrue(coll.size() > 0);
            */

        } catch (JrafPersistenceException e) {
            fail("unexpectedException");
            e.printStackTrace();
        } catch (JrafDaoException e) {
            fail("unexpectedException");
            e.printStackTrace();
        }
    }
}
