package com.airfrance.squaleweb.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import com.airfrance.squalecommon.util.ConstantRulesChecking;
import com.airfrance.squaleweb.resources.WebMessages;

/**
 * Utilitaire pour l'export PDF avec jasperReport
 */
public class JasperReportUtil {
    
    /** Chemin racine de l'application */
    private static final String ROOT = "/squale";
    
    /**
     * @param pLocale la locale
     * @param pTreNames les noms des mtriques
     * @param pValues les valeurs des mtriques
     * @return les mtriques avec les valeurs associes sous forme de chane
     * pour l'afficher dans le PDF 
     */
    public static String getMetrics(Locale pLocale, List pTreNames, List pValues) {
        List valuesStr = SqualeWebActionUtils.getAsStringsList(pValues);
        // La note de la pratique
        String result = WebMessages.getString(pLocale, "export.pdf.mark");
        String mark = SqualeWebActionUtils.formatFloat((String)valuesStr.get(0));
        if(mark == null){
            mark = "";
        }
        result += " : " + mark + "\n";
        // Les mtriques
        int i = 1;
        for(Iterator it = pTreNames.iterator(); it.hasNext(); i++) {
            result += WebMessages.getString(pLocale, (String)it.next()) + "\t:";
            String curValue = (String)valuesStr.get(i);
            // curValue peut tre nul
            if(null == curValue) {
                curValue = "-";
            }
            if(curValue.indexOf("\n") > 0) { // On passe le bloc  la ligne
                result += "\n";
            } else {
                result += "\t";
            }
            result += curValue + "\n";
        }
        return result;
    }

    /**
     * Rcupre le pictogramme associ  la svrit de la rgle
     * @param pSeverity la svrit.
     * @return le chemin de l'image
     */
    public static String generateRulePicto(String pSeverity) {
        String picto = "";
        // Images en fonction de la svrit (error, warning, info)
        final HashMap severities = new HashMap();
        severities.put(ConstantRulesChecking.ERROR_LABEL, "/images/pictos/error.png");
        severities.put(ConstantRulesChecking.WARNING_LABEL, "/images/pictos/warning.png");
        severities.put(ConstantRulesChecking.INFO_LABEL, "/images/pictos/info.png");
        // Vrification simpliste de la svrit pour viter des exceptions
        // lors de sa conversion     
        if (pSeverity != null) {
            picto = (String)severities.get(pSeverity);
            if(null == picto) {
                picto = "/images/pictos/na.gif";
            }
        }
        return picto;
    }
    
    /**
     * @param pNote la note
     * @return le chemin (adapt aux rapports JASPER) vers le pictogramme associ  la note
     */
    public static String generatePicto(String pNote) {
        return SqualeWebActionUtils.IMG[SqualeWebActionUtils.generatePicto(pNote)].replaceAll(ROOT, "");
    }

    /**
    * @param currentMark la note courante
    * @param predecessorMark la note prcdente
    * @return l'image reprsentant l'volution entre currentMark et predecessorMark
    */
    public static String getImageForTrend(String currentMark, String predecessorMark) {
        return SqualeWebActionUtils.getImageForTrend(currentMark, predecessorMark).replaceAll(ROOT, "");
    }
    
    /**
     * @param pText le texte  formater
     * @return le texte sans les balises html
     */
    public static String formatHTML(String pText) {
        String result = "";
        if(null != pText) {
            // on remplace les saut de ligne
            result = pText.replaceAll("<br/?>", "\n");
            // les puces
            result = result.replaceAll("<li>", "\n- ");
            result = result.replaceAll("</li>", "\n");
            // On supprime les autres balises
            result = result.replaceAll("<.+>", "");
        }
        return result;
    }
}
