//Source file: D:\\cc_views\\squale_v0_0_act\\squale\\src\\squalix\\src\\com\\airfrance\\squalix\\core\\Squalix.java

package com.airfrance.squalix.core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.bootstrap.initializer.Initializer;
import com.airfrance.squalix.messages.Messages;

/**
 * Lance l'application Squalix.
 * <br />
 * Ceci consiste en quelques oprations simples :
 * <ul>
 * <li>initialisation du socle JRAF,</li>
 * <li>lecture des paramtres de lancement,</li>
 * <li>instanciation d'un scheduler si l'excution peut-tre lance.</li>
 * </ul>
 * <br />
 * Le lancement du scheduler est synchrone, ce qui bloque l'application tant que le scheduler tourne.
 * <br />
 * Pour plus de renseignements sur le fonctionnement du moteur de tches, reportez-vous  la 
 * javadoc des classes <code>Scheduler</code> et <code>ResourcesManager</code>.
 * 
 * @see com.airfrance.squalix.core.Scheduler
 * @see com.airfrance.squalix.core.ResourcesManager
 * 
 * @author m400842
 * @version 1.0
 */
public class Squalix {

    /**
     * Logger
     */
    private static Log mLOGGER = null;

    /**
     * Cl du site hbergeant l'application
     */
    private static String mSite = null;

    /**
     * Chemin du fichier de configuration
     */
    private static String mConfigFile = null;

    /**
     * Mthode main.
     * @param pArgs arguments de lancement.
     * @roseuid 42918D3702A5
     */
    public static void main(final String pArgs[]) {
        // lancement de linitialisation JRAF
        String rootPath = pArgs[0];
        String configFile = "/config/providers-config.xml";
        Initializer init = new Initializer(rootPath, configFile);
        init.initialize(); 
        // Maintenant que le socle JRAF est initialis, on peut crer un logger
        mLOGGER = LogFactory.getLog(Squalix.class);
        if (null != pArgs && pArgs.length > 0) {
            try {
                // On trie les paramtres
                getParameters(pArgs);
                Scheduler scheduler = null;
                if (null == mSite) {
                    // Si aucun site n'est spcifi, on quitte l'application
                    mLOGGER.fatal(Messages.getString("main.missing_parameters"));
                } else {
                    scheduler = new Scheduler(new Long(mSite).longValue());
                    // on lance le scheduleur
                    scheduler.start();
                }
            } catch (Exception e) {
                // Si un problme apparat dans la rcupration de la configuration
                // ou des paramtres, on loggue et on quitte
                mLOGGER.fatal("Fatal error", e);
            }
        } else {
            mLOGGER.fatal(Messages.getString("main.missing_parameters"));
        }
    }

    /**
     * Rcupre les paramtres de la ligne de commande 
     * @param pArgs liste des paramtres.
     * @roseuid 42CE4F340204
     */
    private static void getParameters(final String pArgs[]) {
        for (int i = 0; i < pArgs.length; i++) {
            if (pArgs[i].equals(Messages.getString("main.site_parameter"))) {
                mSite = pArgs[++i];
            } else if (pArgs[i].equals(Messages.getString("main.configFile_parameter"))) {
                mConfigFile = pArgs[++i];
            }
        }
    }
}
