//Source file: D:\\cc_views\\squale_v0_0_act\\squale\\src\\squalix\\src\\com\\airfrance\\squalix\\core\\TaskUtility.java

package com.airfrance.squalix.core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalecommon.enterpriselayer.businessobject.config.TaskRefBO;
import com.airfrance.squalix.messages.Messages;

/**
 * Cette classe fournit un certin nombre de mthodes "utilitaires" utilisables par les
 * toutes les tches et le moteur.
 * 
 * @author m400842 (by rose)
 * @version 1.0
 */
public class TaskUtility {

    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(TaskUtility.class);

    /**
     * Retourne une instance de la classe de la classe dont le nom
     * est pass en paramtre.
     * 
     * @param pClassName le nom entirement spcifi de la classe.
     * @return l'instance d'une tche.
     * @throws Exception si un problme d'instanciation apparat.
     * @roseuid 42CAA2A1013B
     */
    private static AbstractTask instanciate(String pClassName) throws Exception {
        AbstractTask task = null;
        try {
            Class theClass = Class.forName(pClassName);
            task = (AbstractTask) theClass.newInstance();
        } catch (Exception e) {
            LOGGER.error(e, e);
            LOGGER.error(Messages.getString("exception.task_not_found") + pClassName);
            task = null;
        }
        return task;
    }

    /**
     * Cre un arbre de tches  partir des descripteurs d'excution.<br />
     * Pour chaque tche, une instance est cre, et ses enfants lui sont attribus.<br />
     * Cette action est fait de manire rcursive sur toutes les tches sous la racine du
     * descripteur d'excution. 
     * 
     * @param pTaskRef le nom de la classe de la tache
     * @param pProjectId l'id du projet analys. Peut-tre null.
     * @param pAuditId l'id de l'audit pour lequel est excut la tche. Peut-tre null.
     * @param pApplicationId l'id de l'application contenant ce projet
     * @param pData les paramtres temporaires
     * @return la tche racine.
     * @roseuid 42CD40730176
     */
    public static AbstractTask createTask(TaskRefBO pTaskRef, long pProjectId, long pApplicationId, long pAuditId, TaskData pData) {
        AbstractTask task = null;
        try {
            // Cre une nouvelle instance de tche
            task = instanciate(pTaskRef.getTask().getClassName());
            if (null != task) {
                // On attribue  la tche les valeurs de ses attributs
                // pattern IOC
                task.setProjectId(new Long(pProjectId));
                task.setAuditId(new Long(pAuditId));
                task.setApplicationId(new Long(pApplicationId));
                task.setStatus(AbstractTask.NOT_ATTEMPTED);
                task.setMandatoryTask(pTaskRef.getTask().isMandatory());
                task.setData(pData);
                task.setTaskParameters(pTaskRef.getParameters());
            }
        } catch (Exception e) {
            LOGGER.error(e);
        }
        return task;
    }

    /**
     * Annule la tache qui a chou et celles qui suivaient .<br />
     * Le statut qui leur est attribu est CANCELLED.
     * 
     * @param pTask tche  annuler
     * @roseuid 42D22EF8027A
     */
    public static void stopTask(AbstractTask pTask) {
        if (pTask.getStatus() != AbstractTask.RUNNING) {
            pTask.setStatus(AbstractTask.CANCELLED);
        }
    }

}
