package com.airfrance.squalix.tools.external.bugtracking.qc;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.external.bugtracking.ExtBugTrackingMetricsBO;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.tools.external.bugtracking.AbstractExtBugTrackingTask;

/**
 *
 * 
 */
public class ExtBugTrackingQCTask extends AbstractExtBugTrackingTask {
    
    /**
     * Logger
     */
       private static final Log LOGGER = LogFactory.getLog(ExtBugTrackingQCTask.class);


    /**
     * Constructeur par dfaut
     */
    public ExtBugTrackingQCTask(){
        mName="ExtBugTrackingQCTask";
    }
    
   
    /**
    * C'est cette mthode qui est appel pour l'excution de la tache.   
    * @throws TaskException exception leve par la tache
    */ 
    public void execute() throws TaskException {
        try{
            recup();
            getDefects().setAudit(getAudit());
            getDefects().setComponent(getProject());
            getDefects().setTaskName(getName());     
            MeasureDAOImpl.getInstance().create(getSession(),getDefects());
        }catch (Exception e) {
            throw new TaskException(e);
        }
    }


   /**
    * Cette mthode lance la rcupration des donnes auprs du logiciel externe.
    * Les donnes rcupres sont places dans un objet ExtBugTrackingMetricsBO 
    * @throws TaskException exception leve par la tache
    */
    public void recup() throws TaskException {
        ExtBugTrackingMetricsBO measure = new ExtBugTrackingMetricsBO();
        measure.setNumberOfDefects(getTaskNbDefects());
        measure.setDefectsOpen(getTaskOpen());
        measure.setDefectsAssigned(getTaskAssigned());
        measure.setDefectsTreated(getTaskTreated());
        measure.setDefectsClose(getTaskClose());
        measure.setDefectsAnomaly(getTaskAnomaly());
        measure.setDefectsEvolution(getTaskEvolution());
        measure.setDefectsHigh(getTaskHigh());
        measure.setDefectsMedium(getTaskMedium());
        measure.setDefectsLow(getTaskLow());
        setDefects(measure);        
    }

    
    /**
     * Rcupration auprs du logiciel externe du nombre total de defects
     * @return le nombre total de defects
     */
    public int getTaskNbDefects() {
        int nbDef=200;
        LOGGER.info("");
        return nbDef;
    }


   /**
    * Rcupration auprs du logiciel externe du nombre de defects ouvert
    * @return le nombre de defects ouvert
    */
    public int getTaskOpen() {
        int defOpen=20;
        LOGGER.info("");
        return defOpen;
    }

    /**
     * Rcupration auprs du logiciel externe du nombre de defects assign
     * @return le nombre de defects assigns
     */
    public int getTaskAssigned() {
        int defAssigned=40 ;
        LOGGER.info("");       
        return defAssigned;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects trait
     * @return le nombre de defects traits
     */
    public int getTaskTreated() {
        int defTreated=60;
        LOGGER.info("");
        return defTreated;
    }

    /**
     * Rcupration auprs du logiciel externe du nombre de defects clot
     * @return le nombre de defcts clot
     */
    public int getTaskClose() {
        int defClose=80;
        LOGGER.info("");
        return defClose;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects en volution
     * @return le nombre de defects en volution  
     */  
    public int getTaskEvolution() {
        int defEvolution = 150;
        LOGGER.info("");
        return defEvolution;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects en anomalie
     * @return le nombre de defects en anomalie
     */
    public int getTaskAnomaly() {
        int defAnomaly=50;
        LOGGER.info("");
        return defAnomaly;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects de niveau haut
     * @return le nombre de defects de niveau haut
     */
    public int getTaskHigh() {
        int defHigh=30;
        LOGGER.info("");
        return defHigh;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects de niveau moyen
     * @return le nombre de defcts de niveau moyen 
     */
    public int getTaskMedium() {
        int defMedium=70;
        LOGGER.info("");
        return defMedium;
    }


    /**
     * Rcupration auprs du logiciel externe du nombre de defects de niveau bas
     * @return le nombre de defects de niveau bas
     */
    public int getTaskLow() {
        int defLow=100;
        LOGGER.info("");
        return defLow;
    }

}
