package com.airfrance.squalix.tools.jspvolumetry;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ListParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ParametersConstants;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.StringParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.jspvolumetry.JSPVolumetryProjectBO;
import com.airfrance.squalix.core.AbstractTask;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.util.process.ProcessErrorHandler;
import com.airfrance.squalix.util.process.ProcessManager;
import com.airfrance.squalix.util.process.ProcessOutputHandler;

/**
 */
public class JSPVolumetryTask extends AbstractTask implements ProcessOutputHandler, ProcessErrorHandler {

    /**
     * Constructeur
     */
    public JSPVolumetryTask() {
        mName = "JSPVolumetryTask";
    }

    /** Le processManager **/
    private ProcessManager mProcess;

    /** le nb de jsps */
    private int mNbJsps = 0;

    /** le nombre de lignes de jsps */
    private int mNbJSPLoc = 0;

    /**
     * Logger
    */
    private static final Log LOGGER = LogFactory.getLog(JSPVolumetryTask.class);

    /**
     * La configuration permettant d'avoir les commandes shell pour la volumtrie sur les jsps
     */
    private JSPVolumetryConfiguration mConfiguration;

    /**
     * Prpare l'environnement d'excution de l'analyse :
     * @exception Exception si un probleme d'initialisation apparait
     */
    private void initialize() throws Exception {
        // On rcupre la configuration du module RSM, personnalise
        // avec les paramtres du projet
        mConfiguration = JSPVolumetryConfiguration.build(mProject, JSPVolumetryMessages.getString("configuration.file"), getData());
        File workspace = mConfiguration.getWorkspace();
        workspace.mkdirs();
        //On va vrifier que le workspace est disponible
        if (!workspace.canWrite() || !workspace.canRead()) {
            throw new Exception(JSPVolumetryMessages.getString("exception.no_workspace"));
        }
        LOGGER.info(JSPVolumetryMessages.getString("logs.initialized") + mProject.getParent().getName() + " - " + mProject.getName());
    }

    /**
     * Cre le ProcessManager.
     * On ne fait pas de new mais un set pour implmenter 
     * le pattern IOC pour pouvoir tester sur un environnement windows
     * @param pArguments arguments
     * @param pDirectory rpertoire de lancement
     * @return le ProcessManager normal
     */
    private ProcessManager createProcessManager(String pArguments, File pDirectory) {
        return new ProcessManager(pArguments, null, pDirectory);
    }

    /** 
     * Mthode rcuprant la volumtrie pour les jsps
     * @throws TaskException en cas d'chec
     */
    public void execute() throws TaskException {
        try {
            int execResult = 0;
            initialize();
            //String nbJspsCommand = mConfiguration.getNbJspsCommand();
            //String nbJspsLocCommand = mConfiguration.getNbJspsLocCommand();
            File script = new File(mConfiguration.getScriptPath());
            List jspDir = ((ListParameterBO) (mProject.getParameter(ParametersConstants.JSP))).getParameters();
            String viewPath = (String) mData.getData(TaskData.VIEW_PATH);
            if (jspDir != null) {
                for (int i = 0; i < jspDir.size(); i++) {
                    String jspDirPath = viewPath + ((StringParameterBO) jspDir.get(i)).getValue();
                    // excute le script
                    execResult = process(script.getAbsolutePath() + " " + jspDirPath + " " + mConfiguration.getResultFilePath());
                    //  Les deux rsultats sont inscrits dans le fichier de rsultats
                    parseFile();
                }
            }
            // cre l'objet  persister
            JSPVolumetryProjectBO volumetry = new JSPVolumetryProjectBO();
            // positionne les champs
            volumetry.setComponent(getProject());
            volumetry.setAudit(getAudit());
            volumetry.setJSPsLOC(new Integer(mNbJSPLoc));
            volumetry.setNumberOfJSPs(new Integer(mNbJsps));
            // sauvegarde l'objet
            persists(volumetry);
        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * Parse le fichier pour rcuprer les rsultats
     * @throws NumberFormatException si le rsultat n'est pas un entier
     * @throws IOException si le fichier n'est pas trouv ou si la lecture des rsultats se passe mal
     * @throws TaskException en cas d'chec de rcupration des rsultats
     */
    private void parseFile() throws NumberFormatException, IOException, TaskException {
        File resultFile = mConfiguration.getResultFile();
        if (resultFile.exists()) {
            BufferedReader br = new BufferedReader(new FileReader(resultFile));
            // on ajoute les valeurs obtenus sur les diffrents rpertoires
            // contenant les jsps fourni
            mNbJsps += Integer.parseInt(br.readLine().trim());
            mNbJSPLoc += Integer.parseInt(br.readLine().trim());
            br.close();
        } else {
            throw new TaskException("exception.resultFile.not_exists");
        }

    }

    /**
     * 
     * @param pCommand la commande a excuter
     * @return le rsultat de l'excution de la commande
     * @throws IOException en cas d'che de lecture
     * @throws InterruptedException si le processus est uinterrompu anormalement
     */
    private int process(String pCommand) throws IOException, InterruptedException {
        mProcess = createProcessManager(pCommand, mConfiguration.getWorkspace());
        // On veut grer les informations lances par le processus en sortie
        mProcess.setOutputHandler(this);
        // On cherche  avoir un comportement synchrone pour tre sr de ne pas
        // avoir un tat des donnes incohrent
        return mProcess.startProcess(this);
    }

    /**
     * @param volumetry l'objet  persister
     * @throws JrafDaoException en cas d'chec de la sauvegarde des rsultats
     */
    private void persists(JSPVolumetryProjectBO volumetry) throws JrafDaoException {
        // On ne peut pas utiliser la mthode save (deprecated) on doit utiliser
        // saveAll donc on a besoin d'une collection
        List result = new ArrayList(0);
        result.add(volumetry);
        MeasureDAOImpl.getInstance().saveAll(getSession(), result);
        getSession().commitTransactionWithoutClose();
        getSession().beginTransaction();
    }
    
    /**
     *  
     * {@inheritDoc}
     * @param pOutputLine {@inheritDoc}
     * @see com.airfrance.squalix.util.process.ProcessOutputHandler#processOutput(java.lang.String)
     */
    public void processOutput(String pOutputLine) {
        // TODO Auto-generated method stub
    }
    
    /**
     *  
     * {@inheritDoc}
     * @param pErrorMessage {@inheritDoc}
     * @see com.airfrance.squalix.util.process.ProcessErrorHandler#processError(java.lang.String)
     */
    public void processError(String pErrorMessage) {
        // TODO Auto-generated method stub
    }
}
