package com.airfrance.squalix.tools.mccabe;


import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalix.util.csv.CSVParser;
import com.airfrance.squalix.util.parser.J2EEParser;

/**
 * Excute l'analyse McCabe sur un projet java. L'environnement McCabe doit tre correctement
 * initialis avant le lancement de la tche, ou la commande "cli" appele doit pointer vers
 * un script qui le met en place.<br><br>
 * Il est important que la gnration des rapports dbute par celle du niveau mthode, afin
 * de pouvoir correctement crer les composants ClassBO si ncessaire.<br>
 * L'ordre des rapports dans le configuration est respct.
 * 
 */
public class JavaMcCabeTask extends AbstractMcCabeTask implements CSVParser.CSVHandler {

    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(JavaMcCabeTask.class);

    /**
     * Constructeur
     */
    public JavaMcCabeTask() {
        mName = "JavaMcCabeTask";
    }

    /**
     * {@inheritDoc}
     * On doit utiliser un parser java
     * @see com.airfrance.squalix.tools.mccabe.AbstractMcCabeTask#setParser()
     */
    public void setParser() {
        mParser = new J2EEParser(mProject);
    }
    
    /**
     * {@inheritDoc}
     * @see com.airfrance.squalix.tools.mccabe.AbstractMcCabeTask#setClassTemplate()
     */
    public void setClassTemplate() {
        mClassTemplate = "csv.java.template.class";
    }

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.tools.mccabe.AbstractMcCabeTask#createReport(java.lang.String)
     */
    protected void createReport(String pReport) throws Exception {
        // Gnration du rapport
        super.createReport(pReport);
        // Dans le cas d'un rapport de type classe
        // on va lire ce rapport pour extraire les noms de classe
        if (pReport.startsWith(McCabeMessages.getString("reports.profile.class"))) {
            String fileName = computeReportFileName(pReport);
            LOGGER.info(McCabeMessages.getString("logs.java.class.preprocess", fileName));
            // Lecture du contenu de ce fichier pour en extraire les noms de 
            // classes et les stocker dans le JavaParser
            CSVParser parser = new CSVParser(McCabeMessages.getString("csv.config.file"));
            parser.parseLines(McCabeMessages.getString(mClassTemplate), fileName, this);
        }
    }

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.util.csv.CSVParser.CSVHandler#processLine(java.util.ArrayList)
     */
    public void processLine(List pLine) {
        ((J2EEParser) mParser).addKnownClass((String)pLine.get(0));
    }
}
