package com.airfrance.squaleide.popup.actions;

import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.airfrance.squaleide.util.messages.PluginMessages;

/**
 * Classe mre pour les actions du plugin
 */
public abstract class BasicPlanAction implements IObjectActionDelegate {

    /** le projet li au plan d'action */
    protected IProject project;

    /**
     * Constructeur par dfaut
     */
    public BasicPlanAction() {
        super();
    }

    /** 
     * Recherche le projet li au plan d'action
     * @param action {@inheritDoc}
     * @param targetPart {@inheritDoc}
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        project = null;
        ISelection selection = targetPart.getSite().getSelectionProvider().getSelection();
        if (selection instanceof StructuredSelection) {
            StructuredSelection stSelection = (StructuredSelection) selection;
            // Perspective ressources
            if (stSelection.getFirstElement() instanceof IProject) {
                project = (IProject) stSelection.getFirstElement();
                // Autre perspectives (Java et J2EE)
            } else if(stSelection.getFirstElement() instanceof JavaProject) {
                project = ((JavaProject) stSelection.getFirstElement()).getProject();
            }
        }
        if (project == null) {
            MessageDialog.openError(new Shell(), "Plug-in SqualePlugin", "Aucun project slectionn, Selection : " + selection + "(" + selection.getClass().getName() + ")");
            project = null;
        }
    }

    /** 
     * {@inheritDoc}
     * @param action {@inheritDoc}
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public abstract void run(IAction action);

    /** 
     * {@inheritDoc}
     * @param action
     * @param selection {@inheritDoc}
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
    }
    
    /**
     * Ouvre un message d'erreur
     * @param shell le shell
     * @param key la cl du message
     * @param exception l'exception associe au message
     */
    protected void openError(Shell shell, String key, Exception exception) {
        String title = PluginMessages.getString("dialog.title");
        String message = PluginMessages.getString(key) + exception.getMessage();
        MessageDialog.openError(shell, title, message);
    }

}
