package com.airfrance.squaleide.popup.actions;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.xml.sax.SAXException;

import com.airfrance.squaleide.util.MarkerHelpers;
import com.airfrance.squaleide.xml.IdeActionPlan;
import com.airfrance.squaleide.xml.IdeTransgression;
import com.airfrance.squaleide.xml.ImportIdeActionPlan;

/**
 * Importe un plan d'action SQUALE
 */
public class ImportPlanAction extends BasicPlanAction implements IObjectActionDelegate {

    /**
     * Constructor for Action1.
     */
    public ImportPlanAction() {
        super();
    }

    /** 
     * Importe le plan d'action SQUALE sur le projet
     * @param action {@inheritDoc}
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        Shell shell = new Shell();
        try {
            // Suppresion des marqueurs existant
            MarkerHelpers.removeMarker(project);
            // On rcupre le fichier XML
            IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
            if (window != null) {
                FileDialog fd = new FileDialog(window.getShell());
                fd.open(); //Ouvrir une boite de dialogue
                String path = fd.getFileName();
                // On vrifie que l'utilisateur a bien slectionn un fichier
                if (null != path && path.length() > 0) {
                    path = fd.getFilterPath() + "\\" + path; //Rcuprer le path du fichier
                    // On rcupre l'objet reprsentant le fichier XML
                    IdeActionPlan actionPlan = ImportIdeActionPlan.importConfig(new FileInputStream(path));
                    // On cre une tche par transgression
                    createMarkers(shell, actionPlan.getTransgressions());
                }
            }
            // On affiche l'erreur en ouvrant une bote de dialogue
        } catch (CoreException ce) {
            openError(shell, "import.error", ce);
        } catch (FileNotFoundException fnfe) {
            openError(shell, "import.file_not_found", fnfe);
        } catch (IOException ioe) {
            openError(shell, "import.file.read.error", ioe);
        } catch (SAXException se) {
            openError(shell, "import.parsing.error", se);
        }
    }

    /**
     * @param shell la fentre dans laquelle les marqueurs doivent tre montrs
     * @param transgressions les transgressions du plan d'action
     */
    private void createMarkers(Shell shell, List transgressions) {
        try {
            // On parcours toutes les transgressions dans l'ordre invers pour ajouter les tches
            // car eclipse utilise un pile pour ajouter les marqueurs
            IdeTransgression curT;
            for (int i=transgressions.size()-1; i>=0; i--) {
                curT = (IdeTransgression) transgressions.get(i);
                // 3 svrits possible :
                if (curT.getSeverity().equals(IdeTransgression.ERROR_SEVERITY)) {
                    // Injection d'une erreur
                    MarkerHelpers.createErrorMarker(project, curT.getDescription(), curT.getInFolder(), curT.getResource(), curT.getLocation());
                } else if (curT.getSeverity().equals(IdeTransgression.WARNING_SEVERITY)) {
                    // Injection d'un warning
                    MarkerHelpers.createWarningMarker(project, curT.getDescription(), curT.getInFolder(), curT.getResource(), curT.getLocation());
                } else {
                    // Injection d'une information
                    MarkerHelpers.createInfoMarker(project, curT.getDescription(), curT.getInFolder(), curT.getResource(), curT.getLocation());
                }
            }
        } catch (CoreException ce) {
            openError(shell, "import.error", ce);
        }

    }
}
