package com.airfrance.squaleide.util;

import java.io.File;
import java.io.FileFilter;

/**
 * Filtre de nom utilis pour filtrer les fichiers selon leur extension.
 */
class ExtensionsFileFilter implements FileFilter {

    /** L'extension  filter */
    private String[] mExtensions;

    /**
     * Constructeur par dfaut
     */
    public ExtensionsFileFilter() {
        this(new String[] { ".java", ".jsp" });
    }

    /**
     * Constructeur
     * @param pExtensions les extensions  filtrer
     */
    public ExtensionsFileFilter(String[] pExtensions) {
        mExtensions = pExtensions;
    }

    /**
     * @return les extensions
     */
    public String[] getExtensions() {
        return mExtensions;
    }

    /**
     * Modifie les extensions
     * @param pExtensions les extensions
     */
    public void setExtension(String[] pExtensions) {
        mExtensions = pExtensions;
    }

    /**
     * @param pFile le nom du fichier
     * @return vrai si le fichier doit tre list, faux sinon.
     */
    public boolean accept(File pFile) {
        boolean result = false;
        if (pFile.isFile()) {
            // Si c'est un fichier, on vrifie que le fichier a la bonne extension
            result = checkExtension(pFile);
        } else if (pFile.isDirectory()) { 
        // on ne prend que les rpertoires qui ont des fichiers avec les extensions souhaites
            result = hasFileWithAllowedExtension(pFile);
        }
        return result;
    }

    /**
     * @param pDir le rpertoire de recherche
     * @return true si le dossier contient au moins un fichier avec l'extension autorise
     */
    private boolean hasFileWithAllowedExtension(File pDir) {
        boolean found = false;
        File[] files = pDir.listFiles();
        for(int i=0; i<files.length && !found; i++) {
            if(files[i].isDirectory()) {
                found = hasFileWithAllowedExtension(files[i]);
            } else if(files[i].isFile() && checkExtension(files[i])) {
                return true;
            }
        }
        return found;
    }

    /**
     * Test de l'extension d'un fichier
     * @param pFile fichier  tester
     * @return true si le fichier satisfait aux critres d'extension
     */
    private boolean checkExtension(File pFile) {
        boolean result = false;
        String fileName = pFile.getName();
        int index = fileName.lastIndexOf('.');
        if (index != -1) {
            String extension = fileName.substring(index).toLowerCase();
            // Parcours de chaque extension
            for (int i = 0;(i < mExtensions.length) && (!result); i++) {
                result = extension.equals(mExtensions[i]);
            }
        }
        return result;
    }
}
